"""Check if all dependencies are installed, and try to install them automatically if possible.

On a Windows installation based on OSGeo4W, we run the OSGeo4W installer to install rasterio, geopandas, scipy, etc.
On other systems, we present a warning message and expect the user to take care of this installation.
"""
import os
import sys
if sys.version_info[:2] >= (3, 8):
    # TODO: Import directly (no need for conditional) when `python_requires = >= 3.8`
    from importlib.metadata import PackageNotFoundError, version  # pragma: no cover
else:
    from importlib_metadata import PackageNotFoundError, version  # pragma: no cover


from PyQt5.QtWidgets import QMessageBox, QDialog
from pkg_resources import parse_version
from qgis.core import Qgis, QgsMessageLog

from .pip_install_dialog import PipInstallDialog
from .marvin_qgis_tools import osgeo4w

_inca_wheel = 'inca-0.0.7-py3-none-any.whl'
_inca_url = 'https://marvintest.vito.be/~danckaet'


def install_pip_deps():
    """Install remaining dependencies using pip."""
    install_dialog = PipInstallDialog()
    install_dialog.message.setText('INCA plugin requires the packages Rtree, pyshp and scikit-learn '
                                   '(not part of OSGeo4W), and the inca core package.  OK to download and install?')
    answer = install_dialog.exec()
    if answer != QDialog.Accepted:  # Installation cancelled.
        return False

    proxy_option = ''
    if install_dialog.proxyGroup.isChecked():
        proxy_user = install_dialog.proxyUser.text()
        proxy_pass = install_dialog.proxyPass.text()
        proxy_host = install_dialog.proxyHost.text()
        proxy_port = install_dialog.proxyPort.text()
        if len(proxy_user) and len(proxy_pass):
            auth = f'{proxy_user}:{proxy_pass}@'
        else:
            auth = ''
        proxy_option = f'--proxy http://{auth}{proxy_host}:{proxy_port}'

    # 'python -u': Seems we need to run Python in 'unbuffered' mode to avoid exit status 120
    pip_process = os.popen(f'python -u -m pip install -U {proxy_option} inca@{_inca_url}/{_inca_wheel} 2>&1')
    output = pip_process.read()
    status = pip_process.close()
    if status is not None:
        QMessageBox.warning(None, 'INCA Installation', f'Installation of core INCA package using pip failed.  '
                                                       'INCA may not work correctly\n'
                                                       f'Exit status: {status}, see message log for output.')
        QgsMessageLog.logMessage(f'pip install failed, output: {output}.', level=Qgis.Critical)
        return False
    return True


def check_dependencies():
    """Try to import required extra packages, and try to install them if it fails."""
    # First try to install packages available from OSGeo4W
    if not osgeo4w.check_packages({'geopandas': 'python3-geopandas',
                                   'matplotlib': 'python3-matplotlib',
                                   'openpyxl': 'python3-openpyxl',
                                   'rasterio': 'python3-rasterio',
                                   'rtree': 'python3-rtree',
                                   'sklearn': 'python3-scikit-learn',
                                   'scipy': 'python3-scipy',
                                   'pip': 'python3-pip',
                                   'setuptools': 'python3-setuptools'}):
        return False

    # Now check installed inca version; if inca is not available or not the right version, run pip.
    required_version = _inca_wheel.split('-')[1]
    try:
        installed_version = version('inca')
        QgsMessageLog.logMessage((f'Current inca version: {installed_version}.'), level=Qgis.Info)
        if parse_version(installed_version) >= parse_version(required_version):
            return True # We have everything we need, in the right version.
    except PackageNotFoundError:
        QgsMessageLog.logMessage('No version of inca core package is currently installed.', level=Qgis.Info)

    QgsMessageLog.logMessage(f'inca version {required_version} will be installed using pip.', level=Qgis.Info)
    if not install_pip_deps():
        QMessageBox.warning(None, 'INCA Installation', 'The installation is not complete.  The INCA plugin may not '
                                                       'work correctly.')
        return False

    import inca
    if inca.__version__ != required_version:  # A previous version of inca was already loaded before installation
        QMessageBox.warning(None, 'Restart QGIS', 'The INCA package was updated.  Please restart QGIS before using the '
                                                  'INCA plugin.')
        return False

    return True
